@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301  USA.
  *
  * Author:  Roberto Majadas <roberto.majadas@openshine.com>
  */
@@ -37,6 +37,8 @@
 #define NAUTILUS_SENDTO_LAST_COMPRESS	NAUTILUS_SENDTO_GCONF"/last_compress"
 #define NAUTILUS_SENDTO_STATUS_LABEL_TIMEOUT 10000
 
+#define UNINSTALLED_PLUGINDIR "plugins/.libs"
+
 /* Options */
 static gchar *default_url = NULL;
 static char **filenames = NULL;
@@ -175,82 +177,71 @@ static char *
 pack_files (NS_ui *ui)
 {
 	char *file_roller_cmd;
-	GtkWidget *error_dialog;
+	const char *filename;
+	GList *l;
+	GString *cmd, *tmp;
+	char *pack_type, *tmp_dir, *tmp_work_dir, *packed_file;
 
 	file_roller_cmd = g_find_program_in_path ("file-roller");
-	
-	if (strlen (gtk_entry_get_text(GTK_ENTRY(ui->pack_entry))) != 0)
-	{
-		GList *l;
-		GString *cmd, *tmp;
-		char *pack_type, *tmp_dir, *tmp_work_dir, *packed_file;
+	filename = gtk_entry_get_text(GTK_ENTRY(ui->pack_entry));
 
-		tmp_dir = g_strdup_printf ("%s/nautilus-sendto-%s", 
+	g_assert (filename != NULL && *filename != '\0');
+	
+	tmp_dir = g_strdup_printf ("%s/nautilus-sendto-%s", 
 				   g_get_tmp_dir(), g_get_user_name());	
-		g_mkdir (tmp_dir, 0700);
-		tmp_work_dir = g_strdup_printf ("%s/nautilus-sendto-%s/%li",
-						g_get_tmp_dir(), g_get_user_name(),
-						time(NULL));
-		g_mkdir (tmp_work_dir, 0700);
-		g_free (tmp_dir);
+	g_mkdir (tmp_dir, 0700);
+	tmp_work_dir = g_strdup_printf ("%s/nautilus-sendto-%s/%li",
+					g_get_tmp_dir(), g_get_user_name(),
+					time(NULL));
+	g_mkdir (tmp_work_dir, 0700);
+	g_free (tmp_dir);
 
-		switch (gtk_combo_box_get_active (GTK_COMBO_BOX(ui->pack_combobox)))
-		{
-		case 0:
-			pack_type = g_strdup (".zip");
-			break;
-		case 1:
-			pack_type = g_strdup (".tar.gz");
-			break;
-		case 2: 
-			pack_type = g_strdup (".tar.bz2");
-			break;
-		default:
-			pack_type = NULL;
-			g_assert_not_reached ();
-		}
-
-		gconf_client_set_int(gconf_client, 
-				NAUTILUS_SENDTO_LAST_COMPRESS, 
-				gtk_combo_box_get_active(GTK_COMBO_BOX(ui->pack_combobox)), 
-				NULL);
-
-		cmd = g_string_new ("");
-		g_string_printf (cmd, "%s --add-to=\"%s/%s%s\"",
-				 file_roller_cmd, tmp_work_dir,
-				 gtk_entry_get_text (GTK_ENTRY(ui->pack_entry)),
-				 pack_type);
-
-		/* file-roller doesn't understand URIs */
-		for (l = file_list ; l; l=l->next){
-			char *file;
-
-			file = g_filename_from_uri (l->data, NULL, NULL);
-			g_string_append_printf (cmd," \"%s\"", file);
-			g_free (file);
-		}
-
-		g_spawn_command_line_sync (cmd->str, NULL, NULL, NULL, NULL);
-		g_string_free (cmd, TRUE);
-		tmp = g_string_new("");
-		g_string_printf (tmp,"%s/%s%s", tmp_work_dir,
- 				 gtk_entry_get_text (GTK_ENTRY(ui->pack_entry)),
-				 pack_type);
-		g_free (tmp_work_dir);
-		packed_file = g_filename_to_uri (tmp->str, NULL, NULL);
-		g_string_free(tmp, TRUE);
-		return packed_file;
-	}else{
-		error_dialog = gtk_message_dialog_new (GTK_WINDOW(ui->dialog),
-						       GTK_DIALOG_MODAL,
-						       GTK_MESSAGE_ERROR,
-						       GTK_BUTTONS_CLOSE,
-						       _("You don't insert the package name"));
-		gtk_dialog_run (GTK_DIALOG (error_dialog));
-		gtk_widget_destroy (error_dialog);
-		return NULL;
+	switch (gtk_combo_box_get_active (GTK_COMBO_BOX(ui->pack_combobox)))
+	{
+	case 0:
+		pack_type = g_strdup (".zip");
+		break;
+	case 1:
+		pack_type = g_strdup (".tar.gz");
+		break;
+	case 2: 
+		pack_type = g_strdup (".tar.bz2");
+		break;
+	default:
+		pack_type = NULL;
+		g_assert_not_reached ();
 	}
-	
+
+	gconf_client_set_int(gconf_client, 
+			     NAUTILUS_SENDTO_LAST_COMPRESS, 
+			     gtk_combo_box_get_active(GTK_COMBO_BOX(ui->pack_combobox)), 
+			     NULL);
+
+	cmd = g_string_new ("");
+	g_string_printf (cmd, "%s --add-to=\"%s/%s%s\"",
+			 file_roller_cmd, tmp_work_dir,
+			 filename,
+			 pack_type);
+
+	/* file-roller doesn't understand URIs */
+	for (l = file_list ; l; l=l->next){
+		char *file;
+
+		file = g_filename_from_uri (l->data, NULL, NULL);
+		g_string_append_printf (cmd," \"%s\"", file);
+		g_free (file);
+	}
+
+	g_spawn_command_line_sync (cmd->str, NULL, NULL, NULL, NULL);
+	g_string_free (cmd, TRUE);
+	tmp = g_string_new("");
+	g_string_printf (tmp,"%s/%s%s", tmp_work_dir,
+			 filename,
+			 pack_type);
+	g_free (tmp_work_dir);
+	packed_file = g_filename_to_uri (tmp->str, NULL, NULL);
+	g_string_free(tmp, TRUE);
+	return packed_file;
 }
 
 static gboolean
@@ -260,6 +251,8 @@ status_label_clear (gpointer data)
 	gtk_label_set_label (GTK_LABEL (ui->status_label), "");
 	gtk_widget_hide (ui->status_image);
 
+	ui->status_timeoutid = 0;
+
 	return FALSE;
 }
 
@@ -272,16 +265,13 @@ send_button_cb (GtkWidget *widget, gpointer data)
 	GtkWidget *w;
 
 	gtk_widget_set_sensitive (ui->dialog, FALSE);
-	while (gtk_events_pending ())
-		gtk_main_iteration ();
 
 	p = (NstPlugin *) g_list_nth_data (plugin_list, option);
 	w = (GtkWidget *) g_list_nth_data (ui->contact_widgets, option);
 
 	if (ui->status_timeoutid != 0) {
 		g_source_remove (ui->status_timeoutid);
-		ui->status_timeoutid = 0;
-		status_label_clear (ui->status_label);
+		status_label_clear (ui);
 	}
 
 	if (p == NULL)
@@ -311,7 +301,7 @@ send_button_cb (GtkWidget *widget, gpointer data)
 
 	if (force_user_to_compress){
 		f = pack_files (ui);
-		if (f != NULL){
+		if (f != NULL) {
 			GList *packed_file = NULL;		
 			packed_file = g_list_append (packed_file, f);
 			if (!p->info->send_files (p, w, packed_file)) {
@@ -319,13 +309,14 @@ send_button_cb (GtkWidget *widget, gpointer data)
 				return;
 			}
 			g_list_free (packed_file);
-		}else{
+		} else {
+			gtk_widget_set_sensitive (ui->dialog, TRUE);
 			return;
 		}
 	}else{
 		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ui->pack_checkbutton))){
 			f = pack_files (ui);
-			if (f != NULL){
+			if (f != NULL) {
 				GList *packed_file = NULL;
 				packed_file = g_list_append (packed_file, f);
 				if (!p->info->send_files (p, w, packed_file)) {
@@ -333,11 +324,13 @@ send_button_cb (GtkWidget *widget, gpointer data)
 					return;
 				}
 				g_list_free (packed_file);
-			}else{
+			} else {
+				gtk_widget_set_sensitive (ui->dialog, TRUE);
 				return;
 			}
-		}else{
+		} else {
 			if (!p->info->send_files (p, w, file_list)) {
+				g_list_foreach (file_list, (GFunc) g_free, NULL);
 				g_list_free (file_list);
 				file_list = NULL;
 				return;
@@ -366,15 +359,26 @@ send_if_no_pack_cb (GtkWidget *widget, gpointer data)
 }
 
 static void
-toggle_pack_check (GtkWidget *widget, gpointer data )
+toggle_pack_check (GtkWidget *widget, NS_ui *ui)
 {
 	GtkToggleButton *t = GTK_TOGGLE_BUTTON (widget);
-	NS_ui *ui_x = (NS_ui *) data ;
-	gint toogle ;
+	gboolean enabled, send_enabled;
 
-	toogle = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (t));
-	gtk_widget_set_sensitive (ui_x->pack_combobox, toogle);
-	gtk_widget_set_sensitive (ui_x->pack_entry, toogle);
+	enabled = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (t));
+	gtk_widget_set_sensitive (ui->pack_combobox, enabled);
+	gtk_widget_set_sensitive (ui->pack_entry, enabled);
+
+	send_enabled = TRUE;
+
+	if (enabled) {
+		const char *filename;
+
+		filename = gtk_entry_get_text(GTK_ENTRY(ui->pack_entry));
+		if (filename == NULL || *filename == '\0')
+			send_enabled = FALSE;
+	}
+
+	gtk_widget_set_sensitive (ui->send_button, send_enabled);
 }
 
 static void
@@ -387,6 +391,8 @@ option_changed (GtkComboBox *cb, gpointer data){
 	gtk_widget_hide ((GtkWidget *) aux->data);
 	aux = g_list_nth (ui->contact_widgets, option);
 	gtk_widget_show ((GtkWidget *) aux->data);
+
+	gtk_label_set_mnemonic_widget (GTK_LABEL (ui->send_to_label), aux->data);
 }
 
 static void
@@ -406,7 +412,6 @@ set_contact_widgets (NS_ui *ui){
 		if (GTK_IS_ENTRY (w)) {
 			g_signal_connect (G_OBJECT (w), "activate",
 					G_CALLBACK (send_if_no_pack_cb), ui);
-			gtk_label_set_mnemonic_widget (GTK_LABEL (ui->send_to_label), w);
 		}
 	}
 }
@@ -470,6 +475,26 @@ set_model_for_options_combobox (NS_ui *ui){
 	gtk_combo_box_set_active (GTK_COMBO_BOX (ui->options_combobox), option);
 }
 
+static void
+pack_entry_changed_cb (GObject *object,
+		       GParamSpec *spec,
+		       NS_ui *ui)
+{
+	gboolean send_enabled;
+
+	send_enabled = TRUE;
+
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (ui->pack_checkbutton))) {
+		const char *filename;
+
+		filename = gtk_entry_get_text(GTK_ENTRY(ui->pack_entry));
+		if (filename == NULL || *filename == '\0')
+			send_enabled = FALSE;
+	}
+
+	gtk_widget_set_sensitive (ui->send_button, send_enabled);
+}
+
 static void
 nautilus_sendto_create_ui (void)
 {
@@ -539,6 +564,8 @@ nautilus_sendto_create_ui (void)
 			  G_CALLBACK (send_button_cb), ui);
 	g_signal_connect (G_OBJECT (ui->pack_entry), "activate",
 			  G_CALLBACK (send_button_cb), ui);
+	g_signal_connect (G_OBJECT (ui->pack_entry), "notify::text",
+			  G_CALLBACK (pack_entry_changed_cb), ui);
 
 	if (force_user_to_compress == FALSE){
 		toggle = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (ui->pack_checkbutton));
@@ -563,8 +590,14 @@ nautilus_sendto_plugin_init (void)
 	NstPlugin *p = NULL;
 	gboolean (*nst_init_plugin)(NstPlugin *p);
 	GError *err = NULL;
+	const char *plugindir;
 
-	dir = g_dir_open (PLUGINDIR, 0, &err);
+	if (g_file_test (UNINSTALLED_PLUGINDIR, G_FILE_TEST_IS_DIR) != FALSE)
+		plugindir = UNINSTALLED_PLUGINDIR;
+	else
+		plugindir = PLUGINDIR;
+
+	dir = g_dir_open (plugindir, 0, &err);
 
 	if (dir == NULL) {
 		g_warning ("Can't open the plugins dir: %s", err ? err->message : "No reason");
@@ -577,7 +610,7 @@ nautilus_sendto_plugin_init (void)
 				char *module_path;
 
 				p = g_new0(NstPlugin, 1);
-				module_path = g_module_build_path (PLUGINDIR, item);
+				module_path = g_module_build_path (plugindir, item);
 				p->module = g_module_open (module_path, G_MODULE_BIND_LAZY);
 			        if (!p->module) {
                 			g_warning ("error opening %s: %s", module_path, g_module_error ());
