@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301  USA.
  *
  * Author:  Roberto Majadas <roberto.majadas@openshine.com>
  */
@@ -75,9 +75,15 @@ void contacts_selected_cb (GtkWidget *entry, EContact *contact, const char *iden
 	char *text;
 
 	email = e_contact_get (contact, E_CONTACT_EMAIL_1);
-	name = e_contact_get (contact, E_CONTACT_NAME_OR_ORG);
 
-	text = g_strdup_printf (CONTACT_FORMAT, (char*)e_contact_get_const (contact, E_CONTACT_NAME_OR_ORG), email);
+	name = e_contact_get (contact, E_CONTACT_FULL_NAME);
+	if (name == NULL) {
+		name = e_contact_get (contact, E_CONTACT_NICKNAME);
+		if (name == NULL)
+			name = e_contact_get (contact, E_CONTACT_ORG);
+	}
+
+	text = g_strdup_printf (CONTACT_FORMAT, (char*) name, email);
 	gtk_entry_set_text (GTK_ENTRY (entry), text);
 	g_free (text);
 }
@@ -93,6 +99,12 @@ state_change_cb (GtkWidget *entry, gboolean state, gpointer data)
 	}
 }
 
+static void
+error_cb (EContactEntry *entry_widget, const char *error, NstPlugin *plugin)
+{
+	g_warning ("An error occurred: %s", error);
+}
+
 static void
 add_sources (EContactEntry *entry)
 {
@@ -135,6 +147,8 @@ GtkWidget* get_contacts_widget (NstPlugin *plugin)
 			  G_CALLBACK (contacts_selected_cb), plugin);
 	g_signal_connect (G_OBJECT (entry), "state-change",
 			  G_CALLBACK (state_change_cb), NULL);
+	g_signal_connect (G_OBJECT (entry), "error",
+			  G_CALLBACK (error_cb), plugin);
 
 	add_sources (E_CONTACT_ENTRY (entry));
 	setup_source_changes (E_CONTACT_ENTRY (entry));
